// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

//*************************************************************************
// RAIDAES Routines Interface
// 08/02/2011   Andre Frantzke
// 02/27/2014   Claus Kuch
//*************************************************************************

//******************************************************************************
// includes
#include "cc430x613x_PMA.h"
#include "cc430x613x.h"
#include "global.h"
#include "SPI.h"
#include "RAIDAES.h"
#include "math.h"
#include "UHF_control.h"
#include "timer.h"

//******************************************************************************
// variables
extern char RXData[12];
extern char StatusRead[4];
extern char CRCData[12];
extern char CRCResult[2];
extern char RSSIdata[6];
extern int iMode;
extern double tLow[3];    // array for tLowBit  periods for RF1, RF2, RF3
extern double tHigh[3];   // array for tHighBit periods for RF1, RF2, RF3
extern char TrimValue[3];

volatile unsigned int StartValue;
volatile unsigned int EndValue;
volatile int counter;
volatile int StartCounter;
volatile int EndCounter;

unsigned int	CLK_Measure[100];	// Used by ISR to measure periods
unsigned int	uiTrimFreq[3];		// Trimming results are stored here
unsigned char	ucTrimBytes[3];		// Trimmed byte values are stored here
bool			TA1_timeout;		// overflow flag
unsigned char 	TA1cnt;				// overflow counter


void LF_SET_TrimValue(int Channel, char locTrimValue)
{
    locTrimValue &= 0x7F;  // clear MSB to prevent trim value from locking

    TransceiveSPI (0x06);	// length
    TransceiveSPI (0xF3);   // write address
    TransceiveSPI (0x02);   // write address extension
    switch (Channel)
    {
    	case 1:     TransceiveSPI (0x89);        // enable trim switches at channel 1
                    TransceiveSPI (locTrimValue);// RF1
                    TransceiveSPI (0x00);        // RF2
                    TransceiveSPI (0x00);        // RF3
                    break;
        case 2:     TransceiveSPI (0x8A);        // enable trim switches at channel 2
                    TransceiveSPI (0x00);        // RF1
                    TransceiveSPI (locTrimValue);// RF2
                    TransceiveSPI (0x00);        // RF3
                    break;

        case 3:     TransceiveSPI (0x8C);        // enable trim switches at channel 3
                    TransceiveSPI (0x00);        // RF1
                    TransceiveSPI (0x00);        // RF2
                    TransceiveSPI (locTrimValue);// RF3
                    break;
    }
}


// programs trim value to EEPROM
// depending on channel and trim switch
void ProgTrimSwitch(int Channel, char locTrimValue)
{
    locTrimValue &= 0x7F;  // clear MSB to prevent trim value from locking
    
    TransceiveSPI (0x06);	// length
    TransceiveSPI (0xF3);   // write address
    TransceiveSPI (0x02);   // write address extension
    switch (Channel)
    {
    	case 1:     TransceiveSPI (0x39);        // Prog channel 1
                    TransceiveSPI (locTrimValue);// RF1
                    TransceiveSPI (0x00);        // RF2
                    TransceiveSPI (0x00);        // RF3
                    break;
        case 2:     TransceiveSPI (0x3A);        // Prog channel 2
                    TransceiveSPI (0x00);        // RF1
                    TransceiveSPI (locTrimValue);// RF2
                    TransceiveSPI (0x00);        // RF3
                    break;
    
        case 3:     TransceiveSPI (0x3C);        // Prog channel 3
                    TransceiveSPI (0x00);        // RF1
                    TransceiveSPI (0x00);        // RF2
                    TransceiveSPI (locTrimValue);// RF3
                    break;
    }
   
    RXData[0]=TransceiveSPI(0x00); // get key number
    RXData[1]=TransceiveSPI(0x00); // 0x00
    RXData[2]=TransceiveSPI(0x00); // 0x00
    RXData[3]=TransceiveSPI(0x00); // 0xFF
    RXData[4]=TransceiveSPI(0x00); // 0xFF
    RXData[5]=TransceiveSPI(0x00); // Trim Status
    RXData[6]=TransceiveSPI(0x00); // write address
    RXData[7]=TransceiveSPI(0x00); // write address extension
    
}

void VCL_ON(void)
{
	VCLdiode_ON;
    __delay_cycles(51948); //4ms @ 13MHz
	LED3_ON;
}

void VCL_OFF(void)
{
	VCLdiode_OFF;
    LED3_OFF;
}



int ReadPageBank(int Bank, int Page)
{
	char c_Bank, c_Page;
	c_Bank = (char)Bank;
	c_Page = (char)Page;
    // General Read  Bank 7
    // Command
    TransceiveSPI (0x02);                // length
    TransceiveSPI (c_Bank);              // write address
    TransceiveSPI (c_Page);              // write address extension

    // response
    RXData[0] = TransceiveSPI (0x00);    // key number
    RXData[1] = TransceiveSPI (0x00);    // Page byte 0
    RXData[2] = TransceiveSPI (0x00);    // Page byte 1
    RXData[3] = TransceiveSPI (0x00);    // Page byte 2
    RXData[4] = TransceiveSPI (0x00);    // Page byte 3
    RXData[5] = TransceiveSPI (0x00);    // read address = 0xF1
    RXData[6] = TransceiveSPI (0x00);    // read address extension = page number
    RXData[7] = TransceiveSPI (0x00); 	 // write address extension

    if((RXData[5]==0xF1) && (RXData[6]==c_Page))
    return 1;
	else return 0;
}


// performs short status read, command: 0x00 
void CMDStatusRead(void)
{
    TransceiveSPI (0x00);                   // command: 0x00
    StatusRead[0] = TransceiveSPI (0x00);   // device status
    StatusRead[1] = TransceiveSPI (0x00);   // LF passive mode status
    StatusRead[2] = TransceiveSPI (0x00);   // switch status
    StatusRead[3] = TransceiveSPI (0x00);   // LF passive mode data
}



void CMDRSSI(void)
{
    TransceiveSPI (0x03);                  // length
    TransceiveSPI (0xF3);                  // write address
    TransceiveSPI (0x44);                  // write address extension
    TransceiveSPI (0x00);                  // selects if to start RSSI or take from last measurement
    									   // 0x00 take last measurement during EOF
                                           // 0x01 start RSSI measurement immediatly 
                                           //      (LF field must be present)
                                           // 0x02 automatically start measurement during EOF reception

    RSSIdata[0] = TransceiveSPI (0x00);    // if EOBA=stable=RSSI OK 0x01
    RSSIdata[1] = TransceiveSPI (0x00);    // RSSI value of RF1
    RSSIdata[2] = TransceiveSPI (0x00);    // RSSI value of RF2
    RSSIdata[3] = TransceiveSPI (0x00);    // RSSI value of RF3
    RSSIdata[4] = TransceiveSPI (0x00);    // read address = 0x44
    RSSIdata[5] = TransceiveSPI (0x00);    // read address extension = 0x01
}


void CMDPowerDown(void)
{
    // SPI Power Down 
    TransceiveSPI (0x03);       // length
    TransceiveSPI (0xF3);       // Write Address
    TransceiveSPI (0x41);       // write address extension RAID Control
    TransceiveSPI (0x0F);       // specific
}

// performs Raid Control Command
void RAID_Control(char Command)
{
    /* possible commands
    0Dh  =  00001100:  Power-Down (delayed after MSP-Access-Uplink) 
    0Eh  =  00001110:  Power-Down Command 
    0Fh  =  00001111:  Power-Down with write of Button-Stuck-Mask 
 
    1?h  =  0001??00:  Switch 500us clock to CLK_OUT pin (default). 
    1?h  =  0001??01:  Switch sampling clock switched to CLK_OUT pin 
    1?h  =  0001??10:  Switch immobilizer clock to CLK_OUT pin 
    1?h  =  0001??11:  Switch off CLK_OUT pin 
    1?h  =  000100??:  Switch EOBA to EOB pin (default) 
    1?h  =  000101??:  Switch EOB from Transponder to EOB pin 
    1?h  =  000110??:  Switch Buffer-Read-Request to EOB pin (default) 
    1?h  =  000111??:  Switch off EOB pin 
 
    2?h  = 001011?0:  Immobilizer disable 
    2?h  = 001011?1:  Immobilizer enable (default) 
    2?h  = 0010110?:  WDE disable 
    2?h  = 0010111?:  WDE enable (default) 
 
    FFh = 11111111:   SPI-timeout Reset 
    */
    // SPI Power Down 
    TransceiveSPI (0x03);       // length
    TransceiveSPI (0xF3);       // Write Address
    TransceiveSPI (0x41);       // write address extension RAID Control
    TransceiveSPI (Command);       // specific
}
  

/******************************************************************************
 * - Functions
 ******************************************************************************/
/******************************************************************************
 * @brief	Routine which calculates the fitting Trim Control Byte for SPI LF-
 * 			Trim command, based on given parameters
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	eAnt:	Channel Selection
 * 			eMode:	Write Trim Byte or measure frequency
 ******************************************************************************/
uc GetTrimControl(enum _eANTSEL eAnt, enum _eTRIMMODE eMode)
{
	if(eMode == ProgPerm) // Programs a value permanently as trim byte
	{
		switch(eAnt)
		{
		case ANTALL:
			return (PROG_ONE_ONLY | PROG_EN | ANT1_EN | ANT2_EN | ANT3_EN);	// all three antennas
		case ANT1:
			return (PROG_ONE_ONLY | PROG_EN | ANT1_EN);
		case ANT2:
			return (PROG_ONE_ONLY | PROG_EN | ANT2_EN);
		case ANT3:
			return (PROG_ONE_ONLY | PROG_EN | ANT3_EN);
		default:	// select no antenna
			return 0;
		}
	}
	else // eMode == ProgTemp || Meas     programs temporarily and measures frequency on CLK_OUT
	{
		switch(eAnt)
		{
		case ANTALL:
			return (CLK_OUT_SEL | OSCM_EN | ANT1_EN | ANT2_EN | ANT3_EN);	// not recommended to use
		case ANT1:
			return (CLK_OUT_SEL | OSCM_EN | ANT1_EN);
		case ANT2:
			return (CLK_OUT_SEL | OSCM_EN | ANT2_EN);
		case ANT3:
			return (CLK_OUT_SEL | OSCM_EN | ANT3_EN);
		default:	// select no antenna
			return 0;
		}
	}
}
/******************************************************************************
 * @brief	This function measures the duration of 20 clock cycles of CLK_OUT
 * 			CLK_OUT must be connected to port P2.5

 *			Timer TD1 measures time, based on 10MHz from SMCLK
 *			Timer TD0 counts oscillations of CLK_OUT->TD0CLK
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	none
 ******************************************************************************/
unsigned int Measure(void)
{
    StartValue=0;
    EndValue=0;
    StartCounter=100;
    EndCounter=120;
    counter = 0;

    RAID_Control(0x16);
//	PortMap_LF_Trim();
    __delay_cycles(32000);      //4ms

//	EnableTA0CLK();	// GPIO as clock input from CLK_OUT; Must be mapped as TA0CLK

	CLK_Measure[0] = CLK_Measure[1] = CLK_Measure[2] = 0;

	P3SEL |= BIT5;
	// Setup Timer
    TA1CTL = TASSEL_2 + MC__CONTINUOUS + TACLR;
    TA1CCTL0 = CAP | CCIE | CCIS_0;
    // Activate Timer A0
    TA1R=0;                        			//reset capture compare register value to 0
    TA1CCTL0 |= CCIE;              			// enable timer interrupt
	__bis_SR_register(LPM0_bits + GIE);     // Enter LPM3, enable interrupts
    TA1CCTL0 &= ~CCIE;             			// disable interrupt

//	DisableTA0CLK();
	return CLK_Measure[2] - CLK_Measure[0];
}

/******************************************************************************
 * @brief	Performs a single SPI LF trimming command
 * 			Can be used to either program a certain trim byte, or to measure the
 * 			resonance frequency of this channel (specified by parameters)
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	eAnt	: Channel select
 * 			TrimVal : Value to trim permanently (for actual trimming) or
 * 					  temporarily (for measurement)
 * 			eMode   : Trim or Measure
 ******************************************************************************/
ui TrimStep(enum _eANTSEL eAnt, uc TrimVal, enum _eTRIMMODE eMode)
{
	uc ucArr[7];
	ui uiMeas, uiRetVal;
	uc ucTrimControl = GetTrimControl(eAnt, eMode);
	uc i;

	TrimVal &= 0x7F;	// Lock bits are not accepted by this routine

	ucArr[0] = 6;		// amount of SPI master bytes following
	ucArr[1] = 0xF3;	// TAA
	ucArr[2] = ACTCODE_SPI_LFTRIM;	// action code

	ucArr[3] = ucTrimControl;	// Trimming Control

	ucArr[4] = TrimVal;	// RF1; which values are actually trimmed is determined by Trimming Control
	ucArr[5] = TrimVal; // RF2
	ucArr[6] = TrimVal; // RF3

	for(i=0; i<7; i++)
		TransceiveSPI(ucArr[i]);

	switch(eMode)
	{
	case ProgPerm:
		uiRetVal = ucArr[5];	// return Read Address (contains information if command was executed)
		break;
	default:	// Meas || ProgTemp
		uiMeas = Measure();
		uiRetVal =  uiMeas;
		break;
	}
	for(i=0; i<7; i++)
		ucArr[i] = TransceiveSPI(0x00);
	Timer_Wait_SMCLKcycles(0x1000); 			// Wait for oscillation to settle

	return uiRetVal;
}
/******************************************************************************
 * @brief	Sorts 3-entry array like this
 * 			uiArr[2] > uiArr[1] > uiArr[0]
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	none
 ******************************************************************************/
SortVals(unsigned int* uiArr)
{
	unsigned int uiBuff;

	if(uiArr[0] > uiArr[1])
	{
		uiBuff = uiArr[1];
		uiArr[1] = uiArr[0];
		uiArr[0] = uiBuff;
	}
	if(uiArr[1] > uiArr[2])
	{
		uiBuff = uiArr[2];
		uiArr[2] = uiArr[1];
		uiArr[1] = uiBuff;
	}
	if(uiArr[0] > uiArr[1])
	{
		uiBuff = uiArr[1];
		uiArr[1] = uiArr[0];
		uiArr[0] = uiBuff;
	}
}
/******************************************************************************
 * @brief	Trims a single LF channel via SPI command
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	eChan:			Channel to be trimmed
 	 	 	Returns result of measurement (period, not frequency!)
 ******************************************************************************/
unsigned int TrimChan(enum _eANTSEL eChan)
{
	int TrimSel; 				// Selector for trim switch;
	unsigned int uiMeas[3];		// Contains last three measurements -> median value is used, to improve robustness of algorithm
	uc TrimByte = 0, i, n;

	for(n = 0; n<7; n++)
	{
		TrimSel = 0x40 >> n; // Start with trim capacitor with highest capacitance, use next smaller cap after every iteration
		for(i=0; i<3; i++)
			uiMeas[i] = TrimStep(eChan, TrimByte|TrimSel, Meas);		// measure 3 times to get the median value

		SortVals(uiMeas);				// sort to get the median value at index 1
		if(uiMeas[1] < TARGET_CLKCNT)	// if measured frequency(median) is below target frequency (clocks inverse) then
			TrimByte |= TrimSel;		// add current cap trim switch to final trim byte configuration
	}
	TrimStep(eChan, TrimByte, ProgPerm);	// Permanently program trim byte to found value (permanently != locked)

	if(eChan == ANT1 || eChan == ANT2 || eChan == ANT3)		// save values to global data storage
	{
		ucTrimBytes[eChan] = TrimByte;
		uiTrimFreq[eChan] = uiMeas[1];
	}

	return uiMeas[1];
}
/******************************************************************************
 * @brief	Trims the LF interface using the 7-bit trim-switch of Analog Front End
 *
 * @date	April 2013
 *
 * @version	1.0
 * @param	uiTrimV: Array to store results of trimming
 ******************************************************************************/

void SPI_LFTrim(enum _eANTSEL eAnt, char TrimValue)
{
	TrimStep(eAnt, 0x00, ProgPerm);
	TrimStep(eAnt, TrimValue, ProgPerm);
}

// Timer A0 interrupt service routine
#pragma vector=TIMER1_A0_VECTOR
__interrupt void TIMER1_A0_ISR(void)
{
  __disable_interrupt();    					// disalbe interrupts

  if((counter>=StartCounter) && (counter<EndCounter))
  {
	CLK_Measure[counter-StartCounter] = TA1CCR0;
  }
  else if (counter == EndCounter)
  {
	  __bic_SR_register(LPM0_bits + GIE);       // Enter LPM0, disable interrupts
  }
  else
  {
	  counter++;
  }
  __enable_interrupt(); 						// enable interrupts
}

