// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************
// RAIDAES Routines Interface
// 2.08.2011   Andre Frantzke 
//
//*************************************************************************

//******************************************************************************
// include guard
#ifndef RAIDAES_H_
#define RAIDAES_H_

//******************************************************************************
// prototypes
void ProgTrimSwitch(int Channel, char TrimValue);
void VCL_ON(void);
void VCL_OFF(void);
void LF_SET_TrimValue(int Channel, char TrimValue);
void CMDStatusRead(void);
void CMDRSSI(void);
int ReadPageBank(int Bank, int Page);
void CMDPowerDown(void);
void RAID_Control(char Command);


//******************************************************************************
// defines
											// CRC command, CRC start is normally 0x3791
#define CRC_Start_LowByte 0x91
#define CRC_Start_HighByte 0x37 

											// arbitrary demo fob conventions
#define	ACTCODE_SPI_LFTRIM			0x02	// LF self-Trim via PEPS command
											// SPI LF Trimming Defines
#define	CLK_OUT_SEL					0x80
											//#define	RFU						0x40
#define	PROG_ONE_ONLY				0x20
#define	PROG_EN						0x10
#define	OSCM_EN						0x08
#define	ANT3_EN						0x04
#define	ANT2_EN						0x02
#define	ANT1_EN						0x01

#define TARGET_FREQU			(float) 134.2
#define REF_MEAS_CLK			(float) 10E3
#define	MEAS_CLOCKS				20
#define	TARGET_CLKCNTF			((float) ((REF_MEAS_CLK / TARGET_FREQU) * MEAS_CLOCKS)+0.5)	// +0.5 emulates rounding
#define	TARGET_CLKCNT			(unsigned int) TARGET_CLKCNTF

enum _eTRIMMODE {ProgTemp, ProgPerm, Meas};
enum _eANTSEL {ANT1=0, ANT2=1, ANT3=2, ANTALL};

void SPI_LFTrim(enum _eANTSEL eChan, char TrimValue);

#endif /*RAIDAES_H_*/
