// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

// Wireless Update functions.
// *************************************************************************************************
// 02/27/2014   Claus Kuch
//
//
//*************************************************************************
// includes
#include "RF1A.h"
#include "cc430x613x.h"
#include "global.h"
#include "UHF_control.h"
#include "Timer.h"


//*************************************************************************
// variables
unsigned char packetReceived = 0;
unsigned char packetTransmit = 0;

extern volatile BYTE bUHFDataReceived_event;   		// data received event


unsigned char Strobe(unsigned char strobe)
{
  unsigned char statusByte = 0;
  unsigned int  gdo_state;

  	  	  	  	  	  	  	  	  	  	  	  	  	// Check for valid strobe command
  if((strobe == 0xBD) || ((strobe >= RF_SRES) && (strobe <= RF_SNOP)))
  {

    RF1AIFCTL1 &= ~(RFSTATIFG);						// Clear the Status read flag

    while( !(RF1AIFCTL1 & RFINSTRIFG));				// Wait for radio to be ready for next instruction
    if ((strobe > RF_SRES) && (strobe < RF_SNOP))	// Write the strobe instruction
    {
      gdo_state = ReadSingleReg(IOCFG2);    		// buffer IOCFG2 state
      WriteSingleReg(IOCFG2, 0x29);        			// chip-ready to GDO2

      RF1AINSTRB = strobe;							// write instruction to radio core
      if ( (RF1AIN&0x04)== 0x04 )           		// chip at sleep mode
      {
        if ( (strobe == RF_SXOFF) || (strobe == RF_SPWD) || (strobe == RF_SWOR) ) { }
        else
        {
          while ((RF1AIN&0x04)== 0x04);     		// chip-ready ?
          __delay_cycles(850); 						// 850 = 65us @ 13MHz
        }
      }
      WriteSingleReg(IOCFG2, gdo_state);    		// restore IOCFG2 setting

      while( !(RF1AIFCTL1 & RFSTATIFG) );
    }
    else		                    				// chip active mode (SRES)
    {
      RF1AINSTRB = strobe;
    }
    statusByte = RF1ASTATB;
  }
  return statusByte;
}


unsigned char ReadSingleReg(unsigned char addr)
{
  unsigned char data_out;
  
  	  	  	  	  	  	  	  	  	  	  	  	  	// Check for valid configuration register address,
  if ((addr <= 0x2E) || (addr == 0x3E))				// 0x3E refers to PATABLE
	  	  	  	  	  	  	  	  	  	   	 	 	// Send address + Instruction + 1 dummy byte (auto-read)
    RF1AINSTR1B = (addr | RF_SNGLREGRD);    
  else
	  	  	  	  	  	  	  	  	  	  	  	  	// Send address + Instruction + 1 dummy byte (auto-read)
    RF1AINSTR1B = (addr | RF_STATREGRD);    
  
  while (!(RF1AIFCTL1 & RFDOUTIFG) );
  data_out = RF1ADOUT1B;                    		// Read data and clears the RFDOUTIFG

  return data_out;
}

void WriteSingleReg(unsigned char addr, unsigned char value)
{   
  while (!(RF1AIFCTL1 & RFINSTRIFG));       		// Wait for the Radio to be ready for next instruction
  RF1AINSTRB = (addr | RF_REGWR);					// Send address + Instruction

  RF1ADINB = value; 								// Write data in
  __no_operation(); 
}
        

void ReadBurstReg(unsigned char addr, unsigned char *buffer, unsigned char count)
{
  unsigned int i;
  
  while (!(RF1AIFCTL1 & RFINSTRIFG));       // Wait for INSTRIFG
  RF1AINSTR1B = (addr | RF_REGRD);          // Send addr of first conf. reg. to be read 
                                            // ... and the burst-register read instruction
  for (i = 0; i < (count-1); i++)
  {
    while (!(RFDOUTIFG&RF1AIFCTL1));        // Wait for the Radio Core to update the RF1ADOUTB reg
    buffer[i] = RF1ADOUT1B;                 // Read DOUT from Radio Core + clears RFDOUTIFG
                                            // Also initiates auo-read for next DOUT byte
  }
  buffer[count-1] = RF1ADOUT0B;             // Store the last DOUT from Radio Core
}  


void WriteBurstReg(unsigned char addr,unsigned char *buffer, unsigned char count)
{  
  // Write Burst works wordwise not bytewise - known errata
  unsigned char i;

  while (!(RF1AIFCTL1 & RFINSTRIFG));       // Wait for the Radio to be ready for next instruction
  RF1AINSTRW = ((addr | RF_REGWR)<<8 ) + buffer[0]; // Send address + Instruction

  for (i = 1; i < count; i++)
  {
    RF1ADINB = buffer[i];                   // Send data
    while (!(RFDINIFG & RF1AIFCTL1));       // Wait for TX to finish
  } 
  i = RF1ADOUTB;                            // Reset RFDOUTIFG flag which contains status byte
}


void WritePATable(int iPatable)
{
	//unsigned char valueRead = 0;
    // Write the power output to the PA_TABLE and verify the write operation.

    while( !(RF1AIFCTL1 & RFINSTRIFG));		// wait for radio to be ready for next instruction
    RF1AINSTRW = iPatable; 					// PA Table write (burst)

    while( !(RF1AIFCTL1 & RFINSTRIFG));		// wait for radio to be ready for next instruction
      RF1AINSTR1B = RF_PATABRD;
    while( !(RF1AIFCTL1 & RFDOUTIFG));		// wait for reply of radio core
}


void Transmit(unsigned char *buffer, unsigned char length)
{
  RF1AIES |= BIT9;                          
  RF1AIFG &= ~BIT9;                         // Clear pending interrupts
  RF1AIE |= BIT9;                           // Enable RFIFG9 TX end-of-packet interrupts 
  
  WriteBurstReg(RF_TXFIFOWR, buffer, length);     
  
  Strobe( RF_SNOP );                        // Reset Radio Pointer
  Strobe( RF_STX );                         // Strobe STX
}

void ReceiveOn(void)
{  
  Strobe( RF_SIDLE );						// set UHF transceiver in idle state
  Strobe( RF_SRX );							// enable receive

  RF1AIFG &= ~BIT4;                         // Clear a pending interrupt
  RF1AIE  |= BIT4;                          // Enable the interrupt 
}

void ReceiveOff(void)
{
  RF1AIE &= ~BIT4;                          // Disable RX interrupts
  RF1AIFG &= ~BIT4;                         // Clear pending IFG

  Strobe( RF_SIDLE );						// set UHF transceiver in idle state
  Strobe( RF_SFRX);      					// flush the RX fifo
}


#pragma vector=CC1101_VECTOR
__interrupt void CC1101_ISR(void)
{
   __disable_interrupt();					// disable interrupts
  switch(__even_in_range(RF1AIV,32))        // Prioritizing Radio Core Interrupts
  {
    case  0: break;                         // No RF core interrupt pending
    case  2: break;                         // RFIFG0
    case  4: break;                         // RFIFG1
    case  6: break;                         // RFIFG2
    case  8: break;                         // RFIFG3
    case 10:                                // RFIFG4 - RX end-of-packet
      packetReceived = 1;
  	  Instruction();						// go to Parser
      bUHFDataReceived_event = TRUE;
      break;
    case 12: break;                         // RFIFG5
    case 14: break;                         // RFIFG6
    case 16: break;                         // RFIFG7
    case 18: break;                         // RFIFG8
    case 20: break;                         // RFIFG9 - TX end-of-packet
    case 22: break;                         // RFIFG10
    case 24: break;                         // RFIFG11
    case 26: break;                         // RFIFG12
    case 28: break;                         // RFIFG13
    case 30: break;                         // RFIFG14
    case 32: break;                         // RFIFG15
    default: break;
  }


  __enable_interrupt();						// enable interrupts
}
