// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

//*************************************************************************
// SPI Interface Routines 
// 28.09.2010   Andre Frantzke 
//
//*************************************************************************

//*************************************************************************
// includes
#include "cc430x613x.h"
#include "global.h"


// Init and enable USART0 as SPI-Port 0, 8 Bit, 3-Wire, Master-Mode, f = 500 kHz
void InitInternalSPI (void)
{
    UCA0CTL1 |= UCSWRST;                     	// **Put state machine in reset**, ACLK as clock source
    UCA0CTL0 |= UCMSB + UCMST + UCSYNC;      	// 3-pin, 8-bit SPI master
    UCA0CTL1 |= UCSSEL_2;                    	// SMCLK = 13MHz
    UCA0BR0 = 13;                            	// prescaler = 2 -> SPI clock = SMCLK/2
    UCA0BR1 = 0;
    UCA0CTL1 &= ~UCSWRST;                    	// **Initialize USCI state machine**}
}


// Bidirectional SPI0 transmission, send and receive byte (Master-Mode with BUSY control)
char TransceiveSPI (char txdata)
{
    char rxdata = 0x00;
    while (BUSY_Pin != 0x00)
    {
	#ifndef LEDOFF
    	LED1_ON;
	#else
    	__no_operation();
    	#endif
    }
    LED1_OFF;									// wait until BUSY = 0
    while (!(UCA0IFG&UCTXIFG))
    {
    	#ifndef LEDOFF
    	LED2_ON;
    	#else
    	__no_operation();
    	#endif
    }
    LED2_OFF;	                 				// wait until last SPI-Transmission is finished
    UCA0TXBUF = txdata;                       	// send 8 clocks "txdata" and SPI-clocks
    while (!(UCA0IFG&UCRXIFG))
    {
    	#ifndef LEDOFF
    	LED3_ON;
    	#else
    	__no_operation();
    	#endif
    }
    LED3_OFF;
    											// wait until one byte is received
    rxdata = UCA0RXBUF;                       	// read received data from SPI input buffer
    return (rxdata);							// send "rxdata" to main program
}
