// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

//******************************************************************************
// RF430 Tag Firmware
//
// 02/27/2014   Claus Kuch
//
//******************************************************************************

//******************************************************************************
// includes
#include "Timer.h"
#include "global.h"
#include "cc430x613x.h"
#include "RF1A.h"

//******************************************************************************
// variables
unsigned char ucWaitforTimeout = false;



void InitXT2Osc(void)							// init oscillator with external 26MHz
{
  UCSCTL6 &= ~XT2OFF;                       	// Enable XT2
  UCSCTL3 |= SELREF_2;                      	// FLLref = REFO
                                            	// Since LFXT1 is not used,
                                            	// sourcing FLL with LFXT1 can cause
                                            	// XT1OFFG flag to set
  UCSCTL5 = DIVM__2+ DIVS__2; 					// SMCLK and MCLK = XT2/2

  UCSCTL4 |= SELA__REFOCLK + SELS__XT2CLK 		// ACLK = REFOCLK
		    + SELM__XT2CLK;   					// SMCLK, MCLK = XT2CLK when available, otherwise DCOCLKDIV

  do											// Loop until XT2, XT1 & DCO stabilizes
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG); // Clear XT2,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      	// Clear fault flags
  }while (SFRIFG1&OFIFG);                   	// Test oscillator fault flag


}


void XT2_OFF()
{
    UCSCTL6 |= XT2OFF;                   		// disable XT2
}


void Timer1_Init(void)
{
    TA1CCR1 = 32768-1;							// Set interrupt frequency to 1Hz
    TA1CCTL1 |= CCIE;							// Enable timer interrupt
    TA1CTL |= TASSEL0 + TACLR;					// select clock source to ACLK, timer clear
    											// ACLK stays active in LPM3 Mode
}

void Timer1_Start(void)
{
    TA1CTL |= MC1; 								// Start Timer1 in continuous mode
}


void Timer1_Stop(void)
{
	TA1CTL &= ~MC1;								// exit continous mode
    TA1CCTL1 &= ~CCIE;							// Clear timer interrupt
}


void Timer_Wait_SMCLKcycles(unsigned int uiCycles)
{
	if(uiCycles<1)
		return;

	_BIC_SR(GIE);				       			// clear general interrupt enable bit
	ucWaitforTimeout = true;
	TA0CTL  = TASSEL_2 | ID_0 | MC_0 |TACLR | TAIE;
   	TA0CCR0 = uiCycles;
	TA0CCTL0 |= CCIE;							// Enable Interrupt
	TA0CTL |= MC_2;								// Start counting
	_BIS_SR(LPM3_bits + GIE);					// Set previously cleared general interrupt enable and go to low power mode until interrupt is detected
}


//******************************************************************************
// Interrupt Vectors


#pragma vector=TIMER0_A0_VECTOR					// TimerA0 interrupt service routine
__interrupt void Timer_A_CCIFG_ISR (void)
{
	__disable_interrupt();    					// disalbe interrupts
	if(ucWaitforTimeout != false)
	{
		ucWaitforTimeout = false;
		LPM3_EXIT;
	}
	TA0CTL = 0;									// Timer Halt; Reset Config
	__enable_interrupt(); 						// enable interrupts
}


#pragma vector = TIMER1_A1_VECTOR				// TimerA1 interrupt service routine
__interrupt void TIMER1_A1_ISR(void)
{
	__disable_interrupt();    					// disalbe interrupts
    Timer1_Stop();								// stop timer 1
    TA1CCTL1 &= ~CCIFG;		 					// Reset IRQ flag
	Strobe(RF_SIDLE);							// set UHF transceiver in idle state
	Strobe(RF_SPWD);							// switch uhf transceiver off
    BYPASS_ON;     								// disable TPS62730 before LPM Mode
    XT2_OFF();									// disable XT2 when timeout occured
    __enable_interrupt(); 						// enable interrupts
}

