// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

// Wireless Update functions.
// *************************************************************************************************
// 02/27/2014   Claus Kuch
//
//
// *************************************************************************************************
// Include section
#include <string.h>
#include "stdio.h"
#include "stdlib.h"
#include "math.h"
#include "global.h"
#include "cc430x613x.h"
#include "RF1A.h"
#include "UHF_control.h"
#include "cc430x613x_PMA.h"
#include "cc430x613x_PMM.h"
#include "cc430x613x_ADC12.h"
#include "CC430.h"
#include "RAIDAES.h"
#include "Timer.h"


//******************************************************************************
// variables
extern int cert_confirmation;
extern int rf_flag;
extern int cycle_flag;
extern int iMode;
extern char StatusRead[4];
extern char TrimValue[3];
extern char RSSIdata[6];
extern char RXData[12];
extern int CHN_TRIM[2];
extern int OPER;
extern int IntDegC;
extern int WAKE_EVENT;
extern BYTE voltage;

extern unsigned int	uiTrimFreq[3];			// Trimming results are stored here
extern unsigned char ucTrimBytes[3];		// Trimmed byte values are stored here

char TxBuffer[PACKET_LENGTH + 1];


//******************************************************************************
// select carrier frequency and output power

#ifdef TI_CC_RF_FREQ_868MHz

#define TX_POWER			  "10dBm"
    int iPaTable = 0x7EC2;  				//PATABLE (10 dBm output power)
    int iPaTableSet = 0x7E27;  				//PATABLE (-10 dBm output power)

#endif
#ifdef TI_CC_RF_FREQ_915MHz

    #define TX_POWER			  "-1dBm"
    int iPaTable = 0x7E61;  				//PATABLE (-1 dBm output power)
    int iPaTableSet = 0x7E27;  				//PATABLE (-10 dBm output power)

#endif
#ifdef TI_CC_RF_FREQ_434MHz_m1dBm

	#define TX_POWER			  "-1dBm"
    int iPaTable = 0x7E61;  				//PATABLE (-1 dBm output power)
    int iPaTableSet = 0x7E27;  				//PATABLE (-10 dBm output power)

#endif
#ifdef TI_CC_RF_434MHz_m15dBm

	#define TX_POWER			  "-15dBm"
    int iPaTable = 0x7E1D;                  //PATABLE (-15 dBm output power)
    int iPaTableSet = 0x7E27;               //PATABLE (-10 dBm output power)
#endif
#ifdef TI_CC_RF_434MHz_m20dBm

	#define TX_POWER			  "-20dBm"
    int iPaTable = 0x7E0E;  				//PATABLE (-20 dBm output power)
    int iPaTableSet = 0x7E27;
#endif

//*****************************************************************************
// module global variables

static const unsigned int RX_BUFFER_SIZE = 64;
static const unsigned int TX_BUFFER_SIZE = 64;

#pragma diag_suppress=2170
static BYTE dataRXBuffer[RX_BUFFER_SIZE];
static BYTE dataTXBuffer[TX_BUFFER_SIZE];
#pragma diag_default=2170

static BYTE currentDelimLen;
static BYTE count = 0;
static BYTE dataLength = 0;

static sAdcMeasurement adcData;

//******************************************************************************
// enumerations

enum
{
	STATE_DATA_OFFSET = 128,
	STATE_IDLE = 0,
	STATE_READ,
	STATE_SET,
	STATE_SET_VOLT,
	STATE_TRIM,
	STATE_TRIM_CLEAR,
	STATE_TRIM_READ,
	STATE_TRIM_SET,
	STATE_TRIM_FREQ,
	STATE_READ_MEM = STATE_DATA_OFFSET,
	STATE_TRIM_READ_ANT,
	STATE_TRIM_SET_ANT
};


enum												// parser command table
{
	IDN, SET, VOLT, HIGH, LOW, READ, TRIM, MEM,
	CLEAR, ANT, FREQ, ALL,
	CMD_COUNT,

	PARAM_DATA  = 0xFFF0,
	INVALID_CMD = 0xFFFF
};



static const char* cmdtab[] =
{
	"*IDN?",
	"SET",
	"VOLT",
	"HIGH",
	"LOW",
	"READ",
	"TRIM",
	"MEM",
	"CLE",
	"ANT",
	"FREQ",
	"ALL"
};



static BYTE parserState = STATE_IDLE;
static const char scpiDelimiter[] = ": \n";			// SCPI delimiter

													// define return string for ?IDN command
#pragma diag_suppress=1934
#define IDN_STRING IDN_STRING_ \
	(MODULE, HW_SerNo, GENERAL_SW_VERSION, UNDER_SW_VERSION)
#define IDN_STRING_(mod, ser, major, minor) \
	":" ## mod "#" ## ser ":SV" ## major "." ## minor ";\r"

static const char idnString[] = IDN_STRING;
#pragma diag_default=1934


// *************************************************************************************************
// Global Variable section
int received_package = 0;
int rf_flag = 0;
int cycle_flag = 0;
int cert_confirmation;

extern unsigned char packetTransmit;
extern int iMode;


void Instruction(void)
{
	Timer1_Stop(); 			   		 				// deactivate uhf timeout detection

	if(voltage){BYPASS_OFF;}     					// enable TPS62730 to power device with 2.1V
	int iPageBank[2];
	void* pNextPos;									// pointer for Segment in RX Buffer



		BYTE RxBufferLength;						// length of rx buffer
		BYTE PKTSTAT = 0;							// packet status, GD0x Status

		RxBufferLength = ReadSingleReg( RXBYTES );	// Get received characters count.
		__delay_cycles(120000);             		// 120000 = 9.2 ms @ 13MHz
		P3SEL &=~(BIT3 + BIT2 +BIT1) ;       		// configure P3.1,P3.2,P3.3 as normal Port pins
											 		// this is necessary as enabling of USCI state machine
											 		// can cause the RAIDAES to hang up
		UCA0CTL1 &= ~UCSWRST;                		// **Initialize USCI state machine**
		P3SEL |= 0x0E;                       		// after enabling USCI State Machine
											 	 	// configure P3.1,P3.2,P3.3 as USCI_A0

		CMDStatusRead();         			  		// SPI Status Read


		if(RxBufferLength == 0)				  		// No data received.
		{
			goto Instruction_exit;			  		// skip parser
		}

		PKTSTAT = ReadSingleReg( PKTSTATUS );		// Check the CRC results.
		if(!(PKTSTAT & CRC_OK))						// crc bit cleared when entering rx mode
		{
			OPER = 0x04; 					  		// RX-Mode
			Transmit_Status(OPER);

			goto Instruction_exit;			  		// skip parser
		}


		ReadBurstReg(RF_RXFIFORD, dataRXBuffer + count, RxBufferLength);
													// Copy data into parser buffer
		RxBufferLength = dataRXBuffer[count];		// Get real data length
		memmove(dataRXBuffer + count, dataRXBuffer + count + 1, RxBufferLength);
													// Remove data length from input buffer
		count += RxBufferLength;					// set character count to length of rxbuffer
		dataRXBuffer[count] = 0;					// Mark the end of data.
		Strobe(RF_SIDLE);							// send UHF transceiver in initial condition



	for(;;)											// Parser loop.
	{
		unsigned int cmdID;
		switch(cmdID = GetCmdID())					// Identify command received.
		{
			case IDN:
			{
				if(parserState == STATE_IDLE)
				{
					initRadio();                    // initialize UHF radio

					TxBuffer[0] = strlen(idnString);// determine length of IDN String
					strcpy(TxBuffer+1, idnString);  // copy IDN String into TX Buffer

					Transmit((unsigned char*)TxBuffer, TxBuffer[0] + 1);	// send message
					#ifndef LEDOFF
					FlashLED(1,1);
					#endif
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}


				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				parserState = STATE_IDLE;
				break;
			}

			case SET:								// Trim set command received.
			{
				if(parserState == STATE_IDLE)
				{
					parserState = STATE_SET;
				}
				else if(parserState == STATE_TRIM)
				{
					parserState = STATE_TRIM_SET;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case VOLT:								// Set Voltage command received.
			{
				if(parserState == STATE_SET)
				{
					parserState = STATE_SET_VOLT;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case HIGH:								// Switch to high voltage.
			{
				if(parserState == STATE_SET_VOLT)
				{
					BYPASS_ON;						// disable TPS62730
					voltage = VOLTAGE_HIGH;
					VCLdiode_ON;
					VCLdiode_OFF;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}


				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				parserState = STATE_IDLE;
				break;
			}

			case LOW:								// Switch to low voltage.
			{
				if(parserState == STATE_SET_VOLT)
				{
					BYPASS_OFF;						// enable TPS62730
					voltage = VOLTAGE_LOW;
					VCLdiode_ON;
					VCLdiode_OFF;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}


				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				parserState = STATE_IDLE;
				break;
			}
			case READ:								// Read command received.
			{
				if(parserState == STATE_IDLE)
				{
					parserState = STATE_READ;
				}
				else if(parserState == STATE_TRIM)
				{
					parserState = STATE_TRIM_READ;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case TRIM: 								// Trim command received.
			{
				if(parserState == STATE_IDLE)
				{
					parserState = STATE_TRIM;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}


				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case MEM: 								// Read memory command received
			{
				if(parserState == STATE_READ)
				{
					parserState = STATE_READ_MEM;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}


				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case CLEAR:								// Clear command received.
			{
				if(parserState == STATE_TRIM)
				{
					parserState = STATE_TRIM_CLEAR;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case ANT:								// Antenna command received.
			{
				if(parserState == STATE_TRIM_READ)
				{
					parserState = STATE_TRIM_READ_ANT;
				}
				else if(parserState == STATE_TRIM_SET)
				{
					parserState = STATE_TRIM_SET_ANT;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

													// Remove command from RX buffer.
				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case FREQ:								// Frequency command.
			{
				if(parserState == STATE_TRIM)
				{
					parserState = STATE_TRIM_FREQ;
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
					parserState = STATE_IDLE;
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
													// decrease character count
				break;
			}

			case ALL:								// Clear all trim values / all frequencies.
			{
				if(parserState == STATE_TRIM_CLEAR)
				{
					BYPASS_ON;         				// disable TPS62730
					VCLdiode_ON;       				// charge VCL to VBAT-Vdiode~2.6V
					SPI_LFTrim(ANT1, 0x00);
					SPI_LFTrim(ANT2, 0x00);
					SPI_LFTrim(ANT3, 0x00);
					VCLdiode_OFF;      				// disable VCL charging

					strcpy(TxBuffer+1, ":Done Clear;\r\0");
					TxBuffer[0] = strlen(TxBuffer+1);
					Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
					#ifndef LEDOFF
					FlashLED(1,1);
					#endif
					if(voltage == VOLTAGE_LOW)
						BYPASS_OFF;					// enable TPS62730
				}
				else if(parserState == STATE_TRIM_FREQ)
				{
					BYPASS_ON;         				// disable TPS62730, use full battery voltage
					VCLdiode_ON;       				// charge VCL to VBAT-Vdiode~2.6V
					strcpy(TxBuffer+1,":No Command available!;\r\0");
					VCLdiode_OFF;      				// disable VCL charging
					TxBuffer[0] = strlen(TxBuffer+1);
					Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
					#ifndef LEDOFF
					FlashLED(1,1);
					#endif


					if(voltage == VOLTAGE_LOW)		// Restore DC/DC converter settings.
						BYPASS_OFF;					// enable TPS62730
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}

				memmove(dataRXBuffer, dataRXBuffer + strlen(cmdtab[cmdID]) + currentDelimLen, count);
													// Remove command from RX buffer.
				count -= strlen(cmdtab[cmdID]) + currentDelimLen;
				parserState = STATE_IDLE;
													// decrease character count
				break;
			}

			case PARAM_DATA:						// Parameter or data for preceding command.
			{
				if(parserState == STATE_READ_MEM)	// READ:MEM Bank, Page
				{
					char* pNext;

					iPageBank[0] = strtol((const char*)(dataRXBuffer + currentDelimLen -1), &pNext, 16);
													// Bank
					iPageBank[1] = strtol(pNext+1, 0, 16);
													// Page

					BYPASS_ON;         				// disable TPS62730, use full battery voltage
					VCLdiode_ON;       				// charge VCL to VBAT-Vdiode~2.6V
					CMDStatusRead();    			// reset SPI Time out
					ReadPageBank(iPageBank[0], iPageBank[1]);
					__delay_cycles(65000); 			//~5ms@13MHz

					VCLdiode_OFF;      				// disable VCL charging

					if(voltage == VOLTAGE_LOW)		// Restore DC/DC converter settings
						BYPASS_OFF;					// enable TPS62730

					sprintf(TxBuffer+1,":%d:%d:%d:%d:%d:%d:%d:%d;\r\0",
					RXData[0],
					RXData[1],
					RXData[2],
					RXData[3],
					RXData[4],
					RXData[5],
					RXData[6],
					RXData[7]);

					TxBuffer[0] = strlen(TxBuffer+1);

					Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
					#ifndef LEDOFF
					FlashLED(1,1);
					#endif
				}
				else if(parserState == STATE_TRIM_READ_ANT)
				{
					sprintf(TxBuffer+1,":No Command available!;\r\0");
					TxBuffer[0] = strlen(TxBuffer+1);
					Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);

				}
				else if(parserState == STATE_TRIM_SET_ANT)
				{
					char* pNext;
					iPageBank[0] = strtol((const char*)(dataRXBuffer + currentDelimLen -1), &pNext, 16);
													// determine memory Bank address
					iPageBank[1] = strtol(pNext+1, 0, 16);
													// determine memory Page address

					BYPASS_ON;         				// disable TPS62730, use full battery voltage
					VCLdiode_ON;       				// charge VCL to VBAT-Vdiode~2.6V
					switch (iPageBank[0])
					{
						case 1: SPI_LFTrim(ANT1, iPageBank[1]);
							break;
						case 2: SPI_LFTrim(ANT2, iPageBank[1]);
							break;
						case 3: SPI_LFTrim(ANT3, iPageBank[1]);
							break;
					}

					VCLdiode_OFF;      				// disable VCL charging

					sprintf(TxBuffer+1,":ARF:%d:%d:%d:%d:%d:%d;\r\0",
					ucTrimBytes[0],
					uiTrimFreq[0],
					ucTrimBytes[1],
					uiTrimFreq[1],
					ucTrimBytes[2],
					uiTrimFreq[2]);

					TxBuffer[0] = strlen(TxBuffer+1);
					Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
					#ifndef LEDOFF
					FlashLED(1,1);
					#endif


					if(voltage == VOLTAGE_LOW)		// Restore DC/DC converter settings.
						BYPASS_OFF;					// enable TPS62730
				}
				else
				{
					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}


				memmove(dataRXBuffer, dataRXBuffer + dataLength + currentDelimLen, count);
													// Remove data from RX buffer.
				count -= dataLength + currentDelimLen;

				parserState = STATE_IDLE;
				break;
			}

			default:								// Invalid or incomplete command.
			{
				pNextPos = 0;						// Non null string in RX buffer.
				if(dataRXBuffer[0])
				{									// returns pos of delimiter in RXBuffer
					pNextPos = strpbrk((const char*)(dataRXBuffer + currentDelimLen), scpiDelimiter);
				}

				if(pNextPos)						// when delimiter was found
				{
					int len = strlen(pNextPos);		// Discard invalid command.

					if(len > RX_BUFFER_SIZE)
						len = RX_BUFFER_SIZE;

					memmove(dataRXBuffer, pNextPos, len);
													// Mark end of data.
					dataRXBuffer[len] = 0;
					count = len;

					parserState = STATE_IDLE;

					OPER = 0x04; 					// RX-Mode
					Transmit_Status(OPER);
				}

				else								// Incomplete command received.
				{
					goto Instruction_exit;
				}
			}
		}
	}

    Instruction_exit:
    CMDPowerDown();       							// SPI Power Down

    												// disable UART module to save current
    UCA0CTL1 |= UCSWRST;                      		// **Put state machine in reset**

	#ifndef LEDOFF
  	LED1_ON;                       			  		// enable LED1
    __delay_cycles(500000);                   		// 500000 = 38.5ms @ 13MHz
	LED1_OFF;
	#else
    __delay_cycles(500000);                   		// 500000 = 38.5ms @ 13MHz
	#endif


    Strobe(RF_SIDLE);								// set UHF transceiver in idle state
    Strobe(RF_SPWD);								// switch uhf transceiver off
    XT2_OFF();										// disable XT2 to save energy in LPM Mode
    BYPASS_ON;     									// disable TPS62730 for LPM Mode

  	WAKE_EVENT = 255;                         		// reset WAKE_EVENT
    P1IFG &= ~(BIT0);                         		// clear pending interrupts on P1.0

}


static unsigned int GetCmdID()
{
	currentDelimLen = 1;

													// Expecting data values.
	if(parserState >= STATE_DATA_OFFSET)
	{
													// Space: first parameter, Comma: all other parameters
		if(dataRXBuffer[0] == ' ' || dataRXBuffer[0] == ',')
		{
			BYTE* pDelim = (BYTE*)					// returns address of next found delimiter
				strpbrk((const char*)(dataRXBuffer + currentDelimLen), scpiDelimiter);

			if(pDelim)								// delimiter was found
			{
				dataLength = pDelim - dataRXBuffer - 1;
													// End of command: remove the '\n' after processing.
				if(*pDelim == '\n')
					++currentDelimLen;

				return PARAM_DATA;
			}
		}
	}

	if(dataRXBuffer[0] != ':')						// Command does not start with ':'.
		return INVALID_CMD;

	int i;
	for(i = 0; i < CMD_COUNT; ++i)
	{
													// Compare current command with table.
		int currentCmdLength = strlen(cmdtab[i]);
		if(!memcmp(dataRXBuffer + currentDelimLen, cmdtab[i], currentCmdLength))
		{
			int j = 0;
			for(; j < strlen(scpiDelimiter); ++j)
			{
													// Delimiter found after command -> accept command
				if(dataRXBuffer[currentCmdLength + currentDelimLen] == scpiDelimiter[j])
				{
													// End of command: remove the '\n' after processing.
					if(dataRXBuffer[currentCmdLength + currentDelimLen] == '\n')
					{
						++currentDelimLen;
					}
					return i;
				}
			}
		}
	}


	return INVALID_CMD;								// Command not found in table -> return 0xFFFFFFFF.
}



void initUnModulated(void)
{
	WriteSingleReg(MDMCFG2,  0x30); 				// Modem configuration. ASK/OOK
	WriteSingleReg(FREND0,   0x11); 				// Front end RX configuration.
	WritePATable(iPaTable);
}



void initModulated(void)
{
	WriteSingleReg(MDMCFG2,  0x13); 				// Modem configuration. 2-GFSK, repeated sync word
	WriteSingleReg(FREND0,   0x10); 				// Front end RX configuration.
	WritePATable(iPaTable);
}




void AppendLRC(char startindex, char stopindex)
{
      char i,lrc=0x00;
      for (i=startindex;i<=stopindex;i++)			// Calculates the LRC from startindex to stopindex
      {
            lrc ^= dataTXBuffer[i];					// and apppends it at stopindex+1.
      }
      dataTXBuffer[i] = lrc;
}



void initRadio(void)
{
  writeRFSettings();
  Strobe(RF_SIDLE);

  initModulated();

  WriteSingleReg(PKTCTRL1, 0x04);					// automatic flush of RX Fifo wenn crc failed
  WriteSingleReg(PKTCTRL0, 0x05);					// set synchronous serial mode for rx and tx data, set variable packet length mode
  WriteSingleReg(MDMCFG2,  0x13);					// 30/32 sync word bits detected, set 2-GFSK Modulation
  WriteSingleReg(IOCFG0,   0x06);					// GD0 output control

  Strobe(RF_SIDLE);									// set UHF transceiver in idle state
  Strobe(RF_SXOFF);									// switch uhf transceiver off
}



void Transmit_Status(int OPER)
{
    switch (OPER)
    {
        case 0x01:       							// RX-Mode
          	strcpy(TxBuffer+1,":OPER:RX;\r\0");
          	TxBuffer[0] = strlen(TxBuffer+1);
       		Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
       	    __delay_cycles(120000);              	// 120000 = 9.2 ms @ 13MHz
        	ReceiveOn();
       		break;
        case 0x02:       							// TX-Mode
    	 	{
            	ExecuteADCmeasurement(&adcData);
              	CreateUHFTelegram();
    		}
    		break;
        case 0x03:       							// Trim done

          	strcpy(TxBuffer+1,":OPER:TRIM;\r\0");
          	TxBuffer[0] = strlen(TxBuffer+1);
          	Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
			#ifndef LEDOFF
     		FlashLED(2,1);
			#endif
      		break;
        case 0x04:       							// Fail
          	strcpy(TxBuffer+1,":COMMAND:FAIL;\r\0");
          	TxBuffer[0] = strlen(TxBuffer+1);
          	Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
			#ifndef LEDOFF
     		FlashLED(2,1);
			#endif
     	    initRadio();                     		// initialize UHF radio
      		break;
	}

}



void CreateUHFTelegram()
{
	char StartByte = 0x01;
	char LenByte = 10;
	char CommandByte = 0x01;

													// Generate Packet to transmit
	TxBuffer[1] = StartByte;
	TxBuffer[2] = LenByte;
	TxBuffer[3] = CommandByte;
	TxBuffer[4] = (adcData.voltage >> 8) & 0x00FF;
	TxBuffer[5] = adcData.voltage & 0x00FF;
	TxBuffer[6] = (adcData.temp >> 8) & 0x00FF;
	TxBuffer[7] = adcData.temp & 0x00FF;
	TxBuffer[8] = RSSIdata[1];
	TxBuffer[9] = RSSIdata[2];
	TxBuffer[10] = RSSIdata[3];
	TxBuffer[11] = WAKE_EVENT;
	TxBuffer[12] = '\r';

  	TxBuffer[0] = 12;
    Transmit( (unsigned char*)TxBuffer, TxBuffer[0]+1);
    __delay_cycles(120000);              			// 120000 = 9.2ms @ 13MHz
}


