// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************


//****************************************************************************//
// Function Library for setting the PMA
//
//****************************************************************************//
#include "cc430x613x.h"
#include "cc430x613x_ADC12.h"
#include "cc430x613x_TLV.h"



void ExecuteADCmeasurement(sAdcMeasurement* pAdcData)
{
	volatile float mref,nref;
	volatile long IntDegF, temp, VREFCorrection;

	ADC12CTL0 = 0;								// ADC12 off
  	REFCTL0 = REFMSTR + REFON + REFVSEL_0;  	// Reset REFMSTR to hand over control to
  												// REFVSEL_0 = 1.5V
                                            	// ADC12_A ref control registers enable
 	ADC12CTL0 = ADC12SHT1_12 + ADC12SHT0_12 	// 1024 sample and hold cycles for ADC12MEM0 to ADC12MEM15
 			+ ADC12REFON + ADC12ON + ADC12MSC;	// rising edge, activate ADC12, switch reference on
                                             
    ADC12CTL1 = ADC12SHP + ADC12CONSEQ_1;   	// sampling sequence of channels, enable sample timer
  	ADC12CTL2 = ADC12RES_2 + ADC12SR;        	// 12-bit mode, 50ksps

  	ADC12MCTL0 = ADC12SREF_1 + ADC12INCH_10; 	// VREF+ and V(R-) = AVSS, select Input Channel A2
  	ADC12MCTL1 = ADC12SREF_1 					// VREF+ and V(R-) = AVSS, select Input Channel A3
  			+ ADC12INCH_11 + ADC12EOS;  		// set last conversion


  	// Start conversion
  	ADC12CTL0 |= ADC12ENC;						// ADC12_A conversion enalbe
  	ADC12CTL0 |= ADC12SC;						// ADC12_A sample and conversion start


  	while ((ADC12CTL1 & ADC12BUSY) == ADC12BUSY); // loop while ADC12 is busy

  	//**********************************************************//
  	// Ref Calibration
  	//
  	// Formula see CC430 User Guide  "SLAU259B"
  	// Section : 1.13.4
  	//
  	//**********************************************************//
  	VREFCorrection = ADC12MEM1 * 1.5;
    VREFCorrection = (VREFCorrection * iTLV.REF_15V) >> 16;
    pAdcData->voltage = (int)VREFCorrection;

   	//**********************************************************//
  	// Temperature Sensor Calibration
  	//
  	// Formula see CC430 User Guide  "SLAU259B"
  	// Section : 1.13.4
  	//
  	//**********************************************************//
   	{
   		float fADCref25V85C = iTLV.ADC_25V_Ref_85C;
   		float fADCref25V30C = iTLV.ADC_25V_Ref_30C;

   		mref = ((fADCref25V85C-fADCref25V30C)/(85-30));
   		nref = (iTLV.ADC_25V_Ref_85C-mref*85);
   	}

    VREFCorrection = ADC12MEM0 * 2;
    VREFCorrection = (VREFCorrection * iTLV.REF_15V) >> 16;
    temp = ((VREFCorrection - nref) / mref);

    pAdcData->temp = (int16_t)temp;


	// Shut down ADC12
	ADC12CTL0 &= ~(ADC12ENC | ADC12SC);
	ADC12CTL0 &= ~(ADC12ON + ADC12REFON);
	
	// Shut down reference voltage 	
	REFCTL0 &= ~(REFMSTR + REFVSEL_1 + REFON + REFGENACT);
}
