// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

//****************************************************************************//
// Function Library for setting the PMA
//****************************************************************************//
#include "cc430x613x.h"
#include "cc430x613x_ADC12.h"
#include "global.h"

  #define TI_CC_LED_P1OUT         	P1OUT
  #define TI_CC_LED_P1DIR         	P1DIR
  #define TI_CC_LED_1				BIT1	// LED 1
  #define TI_CC_LED_2				BIT2	// LED 2


void InitPorts(void)
{

	P1DIR = 0xFF;        					// set to high to save current
    P1DIR &=~BIT0;    						// Setup P1.0 as WAKE pin with interrupt capability
    P1IE |= BIT0;                           // P1.0 interrupt enabled
    P1IES = 0x00;                           // P1.0 lo/hi edge
    P1IFG &= ~BIT0;                         // P1.0 IFG cleared

    P1OUT = 0x00 | BIT1 | BIT2 | BIT3 | BIT4 | BIT7 ;

    P2DIR = 0xFF;        					// set to high to save current
    P2DIR |= CLK_OUT_Extern;   				// set Port to see Internal CLK_OUT
    P2OUT = 0x00;        					// set port to output


    // ================================
    // P3.1  SOMI (Slave out Master In)            ==> INPUT
    // P3.2  SIMO (Slave in Master out)            ==> OUTPUT
    // P3.3  SCLK (SPI Clock from Master)          ==> OUTPUT
    // P3.5  CLK_OUT (Clock derived from LF field) ==> INPUT
    // ================================

    P3DIR = 0xFF;        					// set to high to save current
    P3DIR &= ~(BIT5+BIT1);
    P3DIR |= VCLdiode;   					// set pin connected to VCL as output
    P3SEL |=BIT5;
    P3OUT = 0x00;        					// set port to output


    // !!!!!!!! don't configure P3SEL here for SPI communication
	// !!!!!!!! will be done in ISR, to overcome SPI communiction errors


    P4DIR = 0xFF;        					// set to high to save current
    P4DIR &= ~(BIT2 + BIT1);
    P4OUT = 0x00;        					// set port to output

    PJDIR = 0x00;
    PJOUT = 0x00;

    // Setup SPI to RAIDAES
    PMAPPWD = 0x02D52;                      // Get write-access to port mapping regs

    P1MAP3 = PM_NONE;
    P2MAP0 = PM_NONE;

    P3MAP2 = PM_UCA0SIMO;                   // Map UCB0SIMO input to P3.2
    P3MAP1 = PM_UCA0SOMI;                   // Map UCB0SOMI output to P3.1
    P3MAP3 = PM_UCA0CLK;                    // Map UCB0CLK input to P3.3
    P3MAP5 = PM_TA1CCR0A;                   // Map Capture Compare Register to P3.5 to measure CLK_OUT
    PMAPPWD = 0;                            // Lock port mapping registers

}

// *************************************************************************************************
// Function declarations
void ToggleLED(int LED, int count);
void FlashLED(int LED, int count);


// *************************************************************************************************
// @fn          ToggleLED
// @brief       This functions toggle the LED
// @return      none
// *************************************************************************************************
void ToggleLED(int LED, int count)
{
	int i;
	for(i=0;i< count;i++)
	{
		if(LED==1) TI_CC_LED_P1OUT ^= BIT1;
		if(LED==2) TI_CC_LED_P1OUT ^= BIT1;
		__delay_cycles(1000000);
	}
}

// *************************************************************************************************
// @fn          FlashLED
// @brief       This functions toggle the LED
// @return      none
// *************************************************************************************************
void FlashLED(int LED, int count)
{
	int i=0;
	while(i<count)
	{
		if(LED==1)
		{
			TI_CC_LED_P1OUT &= ~TI_CC_LED_1;
			__delay_cycles(600000);
			TI_CC_LED_P1OUT |= TI_CC_LED_1;
			__delay_cycles(600000);
		}
		else if(LED==2)
		{
			TI_CC_LED_P1OUT &= ~TI_CC_LED_2;
			__delay_cycles(600000);
			TI_CC_LED_P1OUT |= TI_CC_LED_2;
			__delay_cycles(600000);
		}
		i++;
	}
}
