// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************


// * ****************************************************************************
// *
// *  Description: The contents of TLV device descriptors are read out using the
// *  core library TLV functions and then stored into global variables.
// *
// *  ACLK = n/a, MCLK = SMCLK = default DCO ~ 1.045MHz
// *
// *                MSP430F55xx
// *             -----------------
// *         /|\|              XIN|-
// *          | |                 |
// *          --|RST          XOUT|-
// *            |                 |
// *            |A10              |
// *
// *   Tyler Witt
// *   Texas Instruments Inc.
// *   October 2011
// *   Built with IAR Embedded Workbench Version: 6.0 & Code Composer Studio V4.2
// ******************************************************************************/
#include "cc430x613x.h"
#include "HAL_TLV.h"
#include "cc430x613x_TLV.h"

// Global Variable section
struct tlv iTLV;

// Die Record Variables
struct s_TLV_Die_Record * pDIEREC;
unsigned char bDieRecord_bytes;
unsigned int temp;
unsigned long WaferID;
unsigned int DieX, DieY, TestRes;

// ADC Variables
struct s_TLV_ADC_Cal_Data * pADCCAL;
unsigned char bADCCAL_bytes;
unsigned int ADCGain, ADCOffset;


// REF Variables
struct s_TLV_REF_Cal_Data * pREFCAL;
unsigned char bREFCAL_bytes;
unsigned int REF_15V, REF_20V, REF_25V;

// Peripheral Variables
struct s_Peripheral_Memory_Data * pPeriph;
unsigned char bPeriph_bytes;
unsigned int Memory_1, Memory_2, Memory_3, Memory_4;
unsigned int PMM_Exist;


/*******************************************************************************
 * void ReadADCCAL(void)
 * 
 * This function will search the TLV block for the ADC CAL tag and store the 
 * address of the first data in the block in the reference pointer passed as an 
 * argument. The data is then saved to local variables.
 ******************************************************************************/

void ReadADCCAL(void)
{
  // Read ADC12 Calibration Values
  Get_TLV_Info(TLV_ADCCAL, 0, &bADCCAL_bytes, (unsigned int **)&pADCCAL);
  // read data from flash
  ADCGain = pADCCAL->adc_gain_factor;
  ADCOffset = pADCCAL->adc_offset;
  iTLV.ADC_15V_Ref_30C = pADCCAL->adc_ref15_30_temp;
  iTLV.ADC_15V_Ref_85C = pADCCAL->adc_ref15_85_temp;
  iTLV.ADC_20V_Ref_30C = pADCCAL->adc_ref20_30_temp;
  iTLV.ADC_20V_Ref_85C = pADCCAL->adc_ref20_85_temp;
  iTLV.ADC_25V_Ref_30C = pADCCAL->adc_ref25_30_temp;
  iTLV.ADC_25V_Ref_85C = pADCCAL->adc_ref25_85_temp;
}

/*******************************************************************************
 * void ReadREFCAL(void)
 * 
 * This function will search the TLV block for the REF CAL tag and store the 
 * address of the first data in the block in the reference pointer passed as an 
 * argument. The data is then saved to local variables.
 ******************************************************************************/

void ReadREFCAL(void)
{
  // Read REF Calibration Values
  Get_TLV_Info(TLV_REFCAL, 0, &bREFCAL_bytes, (unsigned int **)&pREFCAL);
  
  iTLV.REF_15V = pREFCAL->ref_ref15;
  iTLV.REF_20V = pREFCAL->ref_ref20;
  iTLV.REF_25V = pREFCAL->ref_ref25;
}

