// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************

//Global Definitions
#include "cc430x613x.h"

// *************************************************************************************************
// include guard
#ifndef _GlobalDefs
#define _GlobalDefs


//******************************************************************************
// defines and software version
#define PACKET_LEN         (0x05)	    	// PACKET_LEN <= 61
#define PACKET_LENGTH      (0x40)			// 64 byte
#define CRC_LQI_IDX        (PACKET_LEN+2)   // Index of appended LQI, checksum
#define CRC_OK             (BIT7)           // CRC_OK bit

#define MODULE	"RF430F5978EVM"				// Hardware description
#define HW_SerNo	"01"
#define GENERAL_SW_VERSION	    "01"		// Software description
#define UNDER_SW_VERSION		"10"

#define BUFFER_SIZE 50

// *************************************************************************************************
// makros
#define WAKE_A BIT0
#define WAKE_B BIT1

#define UHF_RXMODE BIT0
#define UHF_TXMODE BIT1

#define WAKE                 BIT0
#define WAKE_Pin (P1IN & 0x01)   // Port P1.0 (input) connceted to AFE

#define BUSY_Pin (P4IN & 0x04)   // Port P4.2 (input) connected to AFE
#define EOB_Pin (P4IN & 0x02)    // Port P4.1 (input) connected to AFE
#define EOB                  BIT1

#define SOMI_Pin (P3IN & 0x02)   // Port P3.1 (input) connected to AFE
#define SIMO_Pin (P3IN & 0x04)   // Port P3.2 (output) connected to AFE
#define SCLK_Pin (P3IN & 0x08)   // Port P3.3 (output) connected to AFE
#define CLK_OUT_Pin (P3IN & 0x20)// Port P3.5 (input) connected to AFE 
#define CLK_OUT              BIT5

#define CLK_OUT_Extern       BIT5 //P2.5

// diode connected from P3.7 to VCL
// used for external trimming, needs to be set to high
// to supply LF analog frontend
#define VCLdiode             BIT7
#define VCLdiode_ON          P3OUT |= BIT7
#define VCLdiode_OFF         P3OUT &= ~BIT7

#define LED1                 BIT1
#define LED1_OFF             P1OUT |= BIT1 // high 
#define LED1_ON        	     P1OUT &=~BIT1 // low 
#define LED1_TOGGLE       	 P1OUT ^= BIT1 // toggle

#define LED2                 BIT2
#define LED2_OFF             P1OUT |= BIT2 // high 
#define LED2_ON        	     P1OUT &=~BIT2 // low 
#define LED2_TOGGLE       	 P1OUT ^= BIT2 // toggle

#define LED3                 BIT3
#define LED3_OFF             P1OUT |= BIT3 // high 
#define LED3_ON        	     P1OUT &=~BIT3 // low 
#define LED3_TOGGLE       	 P1OUT ^= BIT3 // toggle

#define LED4                 BIT4
#define LED4_OFF             P1OUT |= BIT4 // high 
#define LED4_ON        	     P1OUT &=~BIT4 // low 
#define LED4_TOGGLE       	 P1OUT ^= BIT4 // toggle

#define SW2                  BIT5
#define SW2_IN               (P1IN & BIT5)

#define BYPASS               BIT2
#define BYPASS_OFF           P2OUT |= BIT2
#define BYPASS_ON            P2OUT &= ~BIT2


#endif

// Trimming via SPI defines

#define	DEBUG_HIGH				{P2OUT |= BIT5; P2DIR |= BIT5;}
#define	DEBUG_LOW				{P2OUT &= ~BIT5; P2DIR |= BIT5;}

#define	VCL_SUPPLY_ON			{P3OUT |= BIT7; P3DIR |= BIT7;}
#define VCL_SUPPLY_OFF			{P3DIR &= ~BIT7; P3OUT &= ~BIT7;}

#define	EnableTA0CLK()			{P3DIR &= ~BIT5; P3SEL |= BIT5;}
#define DisableTA0CLK()			{P3SEL &= ~BIT5;}
#define TA0CLKMapping()			{P3MAP5 = PM_TA0CLK;}

#define false       ((unsigned char)0)
#define true        !false

typedef unsigned char   BYTE;
typedef unsigned char	uc;
typedef unsigned int	ui;
typedef unsigned char	bool;

typedef enum
{
    FALSE,
    TRUE
} tBOOL;

enum
{
	VOLTAGE_HIGH, VOLTAGE_LOW
};

extern BYTE voltage;

