// *************************************************************************************************
//	Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions
//	are met:
//
//		* Redistributions of source code must retain the above copyright
//		notice, this list of conditions and the following disclaimer.
//
//		* Redistributions in binary form must reproduce the above copyright
//		notice, this list of conditions and the following disclaimer in the
//		documentation and/or other materials provided with the distribution.
//
//		* Neither the name of Texas Instruments Incorporated nor the names of
//		its contributors may be used to endorse or promote products derived
//		from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
//	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
//	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// *************************************************************************************************
// 	RF430F5978EVM - Firmware
//
//
//
//	c-kuch@ti.com
//
//	Revision
//		0.00		2012-10-31		[Andre Frantzke]
//		0.01		2013-09-01		change temperature calc [c.kuch]
//      0.02        2014-03-15      Pre release Version [c.kuch]
//      0.03        2014-04-22      change RF delay [c.kuch]
//      0.04        2014-05-08      optimize code [m.maisl]
//      1.00        2014-05-12      First Release [c.kuch]
//		1.10		2015-01-08		low power optimization [d.gerl]
//
//	Plattform: Code Composer Studio  Version: 6.0.1.00040
//******************************************************************************


//******************************************************************************
// standard includes
#include "stdio.h"
#include "string.h"
#include "stdlib.h"

//******************************************************************************
// includes LF Part
#include "global.h"
#include "spi.h"
#include "RAIDAES.h"
#include "cc430x613x_PMA.h"

//******************************************************************************
// includes for UHF part
#include "project.h"
#include "RF1A.h"
#include "cc430x613x.h"
#include "cc430x613x_ADC12.h"
#include "cc430x613x_PMM.h"
#include "cc430x613x_TLV.h"
#include "CC430.h"
#include "UHF_control.h"
#include "Timer.h"

//******************************************************************************
// defines
#define BufferLength 48

//******************************************************************************
// Global Variables
int IntDegC;
int iMode;
int CHN_TRIM[2];

//******************************************************************************
// Global flags set by events
volatile BYTE bUHFDataReceived_event = FALSE;	// Flag set by UHF ISR
int OPER = 0x00;								// 0x00 default value / 0x01 RX-Mode / 0x02 TX-Mode / 0x03 TRIM done
int WAKE_EVENT = 0;								/*
												0 low voltage mode
												1 high voltage mode
												8 Wake A
												9 Wake B
												255 error
												*/
BYTE voltage = VOLTAGE_LOW;

//******************************************************************************
// Variables used for LF Part
char StatusRead[4];
char RSSIdata[6];
char RXData[12];
char CRCData[12];
char CRCResult[2];
double tLow[3];   								// array for tLowBit  periods for RF1, RF2, RF3
double tHigh[3];   								// array for tHighBit periods for RF1, RF2, RF3
char TrimValue[3];

//******************************************************************************

void main(void)
{
	  WDTCTL = WDTPW+WDTHOLD;                   // Stop WDT
	  InitPorts();                     			// Port directions, peripherals, portmapping
	  	  	  	  	  	  	  	  	  	  	  	// declare wake interrupt
	  SetVCore(2);                     			// set supply voltage

	  initRadio();            					// initialize UHF radio
	  InitXT2Osc();								// initialize XT2
	  	  	  	  	  	  	  	  	  	  	  	// wait for oscillator init
	  __delay_cycles(50000);					// 50000 = 3.8ms @ 13MHz
	  Strobe(RF_SIDLE);							// set UHF transceiver in idle state
	  Strobe(RF_SPWD);							// switch uhf transceiver off
	  XT2_OFF();								// disable XT2 to save energy in LPM Mode
	  InitInternalSPI();               			// Enable internal SPI to RAIDAES
	  ReadADCCAL();					 			// read ADC calibration data from flash memory
	  ReadREFCAL();					 			// read the reference voltage value from flash memory

	  FlashLED(1,1);					 		// flash LED to singalize initalization

	  BYPASS_ON;
	  while(1)
	  {
  	  __bis_SR_register(GIE+ LPM4_bits);		// Enter LPM4
  	  __no_operation();                         // For debugger
	  }
}   

// Port 1 interrupt service routine
#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
	__disable_interrupt();				 		// disable interrupts during ISR
	P1IFG &= ~BIT0;                     		// clear pending interrupts on P1.0
	if(voltage){BYPASS_OFF;}     		 		// enable TPS62730 to power device with 2.1V
	int i;
    IntDegC = 220;					 			// set initial temperature to 22C
    InitXT2Osc();   	            			// initialize XT2
    											// wait for oscillator init
    __delay_cycles(25000);            			// 25000 = 1.9ms @ 13MHz

    for(i=0;i<12;i++){RXData[i]=0x00;}		 	// clear RXData array
    for(i=0;i<6;i++){RSSIdata[i]=0xFF;}			// clear RSSIdata array
    for(i=0;i<4;i++){StatusRead[i] = 0x00;}	 	// clear StatusRead array

	 	 	 	 	 	 	 	 	 	 		// wait for wake pattern and RSSI measurement to end
    P3SEL &=~(BIT3 + BIT2 +BIT1) ;       		// configure P3.1,P3.2,P3.3 as normal Port pins
                                         		// this is necessary as enabling of USCI state machine
                                         		// can cause the RAIDAES to hang up
    UCA0CTL1 &= ~UCSWRST;                		// **Initialize USCI state machine**
    P3SEL |= 0x0E;                       		// after enabling USCI State Machine
                                         		// configure P3.1,P3.2,P3.3 as USCI_A0

    CMDStatusRead();      			 			// SPI Status Read

     											// check which Wake event happened and set variable WAKE_EVENT
    if((StatusRead[0]&BIT0) == WAKE_A)			// Wake pattern A
    {
    	if ((StatusRead[3]&BIT0) == UHF_RXMODE) // Wake up MRD2 Command => 0108C8A000000108880DEC
        {
        	WAKE_EVENT = 8;						// 8 = uhf mode
            CMDStatusRead();    				// reset SPI Time out
            __delay_cycles(30000); 				// 30000 = 2.3ms @ 13MHz
            CMDRSSI(); 							// perform RSSI measurement
        	OPER = 0x01; 						// RX-Mode
        	Transmit_Status(OPER);
        }
        else if ((StatusRead[3]&BIT1) == UHF_TXMODE) // Wake up MRD2 Command => 0108C8A000000108480D2C
        {
        	WAKE_EVENT = 8;						// 8 = uhf mode
            CMDStatusRead();    				// reset SPI Time out
            __delay_cycles(30000); 				// 30000 = 2.3ms @ 13MHz
            CMDRSSI(); 							// perform RSSI measurement
        	OPER = 0x02; 						// TX-Mode
        	Transmit_Status(OPER);
        }
    }

    else if ((StatusRead[0]&BIT1) == WAKE_B) 	// Wake pattern B
    {
    	WAKE_EVENT = 9;
    	CMDRSSI();               				// perform RSSI measurement
    }
    else
    {
        switch (StatusRead[2])
        {
            case 0x01:  WAKE_EVENT = 0;  		// 0 = low voltage mode
                        BYPASS_ON;
                        VCLdiode_ON;
                        voltage = VOLTAGE_HIGH;
                        break;
            case 0x02:  WAKE_EVENT = 1; 		// 1 = high voltae mode
                        BYPASS_OFF;
                        VCLdiode_ON;
                        voltage = VOLTAGE_LOW;
                        break;

	        default:    WAKE_EVENT = 255; 		// 255 = error
                        break;
	    }
 	}





    for(i=0;i<4;i++){StatusRead[i] = 0x00;}		// clear StatusRead array

    CMDPowerDown();       						// SPI Power Down
        										// disable UART module to save current
    UCA0CTL1 |= UCSWRST;                    	// **Put state machine in reset**

    WAKE_EVENT = 255;   						// reset WAKE_EVENT



    if(OPER != 0x01)							// do not shut down XT2 when RX Mode is enabled
    {
    	Strobe(RF_SIDLE);						// set UHF transceiver in idle state
    	Strobe(RF_SPWD);						// switch uhf transceiver off
      	XT2_OFF();								// disable XT2 to save energy in LPM Mode
    	BYPASS_ON;     							// disable TPS62730 for LPM Mode
    }
    else										// missing uhf signal in RX mode avoids shutting down xt2
    {
    	Timer1_Init();							// init timer for missing uhf timeout
  		Timer1_Start();							// start timer
    }

    __enable_interrupt();						// enable interrupts after ISR

}




